<?php

namespace Drupal\custom_elements\Plugin\CustomElementsFieldFormatter;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemInterface;

/**
 * Implementation of the 'file' custom element formatter plugin.
 *
 * File/image fields are a combination of a reference to a file/image entity,
 * and several extra properties. To set those properties into attributes, this
 * class extends FlattenedCeFieldFormatter. The referenced file is not
 * fieldable / does not have its own CE display, so its properties are added
 * using custom logic by this class. Extra file properties may be added as the
 * need arises.
 *
 * @CustomElementsFieldFormatter(
 *   id = "file",
 *   label = @Translation("File properties")
 * )
 */
class FileCeFieldFormatter extends FlattenedCeFieldFormatter {

  /**
   * {@inheritdoc}
   */
  public static function isApplicable(string $plugin_id, FieldDefinitionInterface $field_definition) {
    return in_array($field_definition->getType(), ['file', 'image'], TRUE);
  }

  /**
   * {@inheritdoc}
   */
  protected function getFieldValues(FieldItemInterface $field_item): array {
    $values = parent::getFieldValues($field_item);
    unset($values['target_id']);
    // Add url property of the referenced file to the values.
    $values['url'] = $field_item->entity->uri->url;
    return $values;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    // target_id is the 'main' property, so nothing gets set into a slot.
    $summary[] = $this->t('Is Slot is ignored.');
    return $summary;
  }

}
