<?php

namespace Drupal\custom_elements\Plugin\CustomElementsFieldFormatter;

use Drupal\Component\Render\MarkupInterface;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\custom_elements\CustomElement;
use Drupal\custom_elements\CustomElementsFieldFormatterBase;

/**
 * Implementation of the 'raw' custom element formatter plugin.
 *
 * @CustomElementsFieldFormatter(
 *   id = "raw",
 *   label = @Translation("Raw"),
 *   weight = -10
 * )
 */
class RawCeFieldFormatter extends CustomElementsFieldFormatterBase {

  /**
   * {@inheritdoc}
   */
  public function build(FieldItemListInterface $items, CustomElement $custom_element, $langcode = NULL) {
    $this->processRaw($items, $custom_element, $this->isSlot(), $this->getName());
  }

  /**
   * Fetch raw field items data and adds to the custom element.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface $field_items
   *   Field item list object.
   * @param \Drupal\custom_elements\CustomElement $custom_element
   *   The custom element to which to add it.
   * @param bool $is_slot
   *   Whether it should be rendered as slot.
   * @param string $element_name
   *   Configured element name.
   */
  protected function processRaw(FieldItemListInterface $field_items, CustomElement $custom_element, bool $is_slot, string $element_name) {
    $values = $field_items->getValue();
    // Make single-value fields flat by default.
    if ($field_items->getFieldDefinition()->getFieldStorageDefinition()->getCardinality() == 1) {
      $values = reset($values);
    }
    if ($is_slot) {
      $custom_element->addSlot($element_name, is_scalar($values) || $values instanceof MarkupInterface ? $values : Json::encode($values));
    }
    else {
      $custom_element->setAttribute($element_name, $values);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    // Nothing needed.
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    // Nothing needed.
  }

}
